'----------------------------------------------------------------------------------------------------------------------
'
' In place Radix-2 Decimation In Time FFT
' Michael Rychlik. 2010-12-28
'
' This FFT was developed from the description by Douglas L. Jones at
' http://cnx.org/content/m12016/latest/.
' It is written as a direct implementation of the discussion and diagrams on that page
' with an emphasis on clarity and ease of understanding rather than speed.
'
' There are two implementations here. The initial version in Spin and a PASM version derived from it.
' Use #define SPIN_BUTTERFLIES or PASM_BUTTERFLIES to select them
'
' The PASM verson does a 1024 FFT in 40ms.
'
' WARNING: I have written this, my first ever FFT implememtation, as a challenge to myself to understand
' Cooley-Tookey from the maths to the code without peeking at the example code on the above page or any other.
' So be warned it may not be accurate.
'
' A big thank you to Dave Hein for clearing up some issues during a great FFT debate on
' the Parallax Inc Propller discussion forum:
' http://forums.parallax.com/showthread.php?127306-Fourier-for-dummies-under-construction
'
' Thanks to Lonesock for the multiply routines.
'
' 2010-12-16    v0.1  Initial draft.
'
' 2010-12-18    v0.2  First working version.
'
' 2010-12-18    v0.3  Optimized butterfly from 4 multiplies to 3
'                     Reduced twiddle tables from longs to words
'
' 2010-12-21    v0.4  A few simple optimizations
'                     First PASM implementation.
'                     Split out test harness.
'
' 2010-12-28    v0.5  Added getsin and getcos fuctions, not taken into use yet.
'
'
'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
CON
    'Specify size of FFT buffer here with length and log base 2 of the length.
    'N.B. Changing this will require changing the "twiddle factor" tables.
    '     and may also require changing the fixed point format (if going bigger)
    FFT_SIZE      = 1024
    LOG2_FFT_SIZE = 10


#define USE_FASTER_MULT
'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
VAR
    'cos and sin parts of the signal to be analysed
    'Result is written back to here.
    'Just write input sammles to bx and zero all by.
    long bx[FFT_SIZE]
    long by[FFT_SIZE]

    long runFlag
'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
PUB start
#ifdef PASM_BUTTERFLIES
    runFlag := 0
    bx_ptr := @bx[0]
    by_ptr := @by[0]
    wx_ptr := @wx[0]
    wy_ptr := @wy[0]
    runFlag_ptr := @runFlag
    cognew (@bfly, 0)     'Check error
#endif

PUB getbx
    return @bx

PUB getby
    return @by

'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
PUB decimate | i, revi, tx1, ty1, tz1
'Radix-2 decimation in time.
'Moves every sample of bx and by to a postion given by
'reversing the bits of its original array index.
    repeat i from 0 to FFT_SIZE - 1
        revi := i >< LOG2_FFT_SIZE
        if i < revi
            tx1 := bx[i]
            ty1 := by[i]

            bx[i] := bx[revi]
            by[i] := by[revi]

            bx[revi] := tx1
            by[revi] := ty1
'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
#ifdef SPIN_BUTTERFLIES
PUB butterflies | k1, k2, k3, a, b, c, d, butterflyMax, groupMax, levelMax, butterflySpan, {
}groupSize, noGroups, b0, b1, wIndex, level, group, butterfly, groupIndex, tx, ty
'Apply FFT butterflies to N complex samples in buffers bx and by, in time decimated order!
'Resulting FFT is produced in bx and by in the correct order.
    butterflySpan := 1
    groupSize := 1
    noGroups := FFT_SIZE >> 1

    'Loop though the decimation levels
    levelMax := LOG2_FFT_SIZE - 1
    repeat level from 0 to levelMax
        groupIndex := 0
        'Loop through each group on a level.
        groupMax := noGroups - 1
        repeat group from 0 to groupMax
            wIndex := 0
            'Loop through butterflies within a group.
            butterflyMax := groupSize - 1
            repeat butterfly from 0 to butterflyMax
                b0 := groupIndex + butterfly
                b1 := b0 + butterflySpan

                'At last...the butterfly.
                a := bx[b1]                 'Get X[b1]
                b := by[b1]

                c := ~~wx[wIndex]           'Get W[wIndex]
                d := ~~wy[wIndex]

                k1 := (a * (c + d)) / 4096  'Somewhat optimized complex multiply
                k2 := (d * (a + b)) / 4096  '    T = X[b1] * W[wIndex]
                k3 := (c * (b - a)) / 4096
                tx := k1 - k2
                ty := k1 + k3

                k1 := bx[b0]
                k2 := by[b0]
                bx[b1] := k1 - tx           'X[b1] = X[b0] * T
                by[b1] := k2 - ty

                bx[b0] := k1 + tx           'X[b0] = X[b0] * T
                by[b0] := k2 + ty

                wIndex += noGroups
            'endRepeat
            groupIndex += (groupSize << 1)
        'endRepeat
        butterflySpan <<= 1
        groupSize <<= 1
        noGroups >>= 1
    'endRepeat
'endPub
'----------------------------------------------------------------------------------------------------------------------
#endif
'----------------------------------------------------------------------------------------------------------------------
#ifdef PASM_BUTTERFLIES
PUB butterflies
    runFlag := 1
    repeat while runFlag <> 0

'----------------------------------------------------------------------------------------------------------------------
DAT
              org       0
'Apply FFT butterflies to N complex samples in buffers bx and by, in time decimated order!
'Resulting FFT is produced in bx and by in the correct order.
'This is a direct translation from the Spin code above, original Spin code in comments.
bfly          rdlong    a, runFlag_ptr wz
        if_z  jmp       #bfly

              mov       butterflySpan, #1                  'butterflySpan := 1
              mov       groupSize, #1                      'groupSize := 1
              mov       noGroups, noGroupsInit             'noGroups := FFT_SIZE >> 1

              mov       levelMax, #LOG2_FFT_SIZE - 1       'levelMax := LOG2_FFT_SIZE - 1

              mov       level, #0                          'repeat level from 0 to levelMax
:l_loop
              mov       groupIndex, #0                     'groupIndex := 0

              mov       groupMax, noGroups                 'groupMax := noGroups - 1
              sub       groupMax, #1

              mov       group, #0                          'repeat group from 0 to groupMax
:g_loop
              mov       wIndex, #0                         'wIndex := 0

              mov       butterflyMax, groupSize            'butterflyMax := groupSize - 1
              sub       butterflyMax, #1

              mov       butterfly, #0                      'repeat butterfly from 0 to butterflyMax
:b_loop
              mov       b0, groupIndex                     'b0 := groupIndex + butterfly
              add       b0, butterfly

              mov       b1, b0                             'b1 := b0 + butterflySpan
              add       b1, butterflySpan

              mov       index, b1                          'a := bx[b1]
              shl       index, #2
              mov       hub_ptr, bx_ptr
              add       hub_ptr, index
              rdlong    a, hub_ptr
              mov       hub_ptr, by_ptr                    'b := by[b1]
              add       hub_ptr, index
              rdlong    b, hub_ptr

              mov       index, wIndex                      'c := ~~wx[wIndex]
              shl       index, #1
              mov       hub_ptr, wx_ptr
              add       hub_ptr, index
              rdword    c, hub_ptr
              shl       c, #16                             'Sign extend
              sar       c, #16

              mov       hub_ptr, wy_ptr                    'd := ~~wy[wIndex]
              add       hub_ptr, index
              rdword    d, hub_ptr
              shl       d, #16                             'Sign extend
              sar       d, #16

              mov       m1, c                              'k1 := (a * (c + d)) / 4096
              add       m1, d
              mov       m2, a
              call      #mul
              mov       k1, m1
              sar       k1, #12

              mov       m1, a                              'k2 := (d * (a + b)) / 4096
              add       m1, b
              mov       m2, d
              call      #mul
              mov       k2, m1
              sar       k2, #12

              mov       m1, b                              'k3 := (c * (b - a)) / 4096
              sub       m1, a
              mov       m2, c
              call      #mul
              mov       k3, m1
              sar       k3, #12

              mov       tx, k1                             'tx := k1 - k2
              sub       tx, k2

              mov       ty, k1                             'ty := k1 + k3
              add       ty, k3

              mov       index, b0                          'k1 := bx[b0]
              shl       index, #2
              mov       hub_ptr, bx_ptr
              add       hub_ptr, index
              rdlong    k1, hub_ptr

              mov       hub_ptr, by_ptr                    'k2 := by[b0]
              add       hub_ptr, index
              rdlong    k2, hub_ptr

              mov       a, k1                              'X[b1] = X[b0] * T   'bx[b1] := k1 - tx
              sub       a, tx
              mov       index, b1
              shl       index, #2
              mov       hub_ptr, bx_ptr
              add       hub_ptr, index
              wrlong    a, hub_ptr

              mov       a, k2                              'by[b1] := k2 - ty
              sub       a, ty
              mov       index, b1
              shl       index, #2
              mov       hub_ptr, by_ptr
              add       hub_ptr, index
              wrlong    a, hub_ptr

              mov       a, k1                              'X[b0] = X[b0] * T    'bx[b0] := k1 + tx
              add       a, tx
              mov       index, b0
              shl       index, #2
              mov       hub_ptr, bx_ptr
              add       hub_ptr, index
              wrlong    a, hub_ptr

              mov       a, k2                              'by[b0] := k2 + ty
              add       a, ty
              mov       index, b0
              shl       index, #2
              mov       hub_ptr, by_ptr
              add       hub_ptr, index
              wrlong    a, hub_ptr

              add       wIndex, noGroups                   'wIndex += noGroups

              cmp       butterfly, butterflyMax wz         'End of butterfly loop
        if_nz add       butterfly, #1
        if_nz jmp       #:b_loop

              mov       tx, groupSize                      'groupIndex += (groupSize << 1)
              shl       tx, #1
              add       groupIndex, tx

              cmp       group, groupMax wz
        if_nz add       group, #1                          'End of group loop
        if_nz jmp       #:g_loop

              shl       butterflySpan, #1                  'butterflySpan <<= 1
              shl       groupSize, #1                      'groupSize <<= 1

              shr       noGroups, #1                       'noGroups >>= 1

              cmp       level, levelMax wz
        if_nz add       level, #1                          'End level loop
        if_nz jmp       #:l_loop

              mov       a, #0
              wrlong    a, runFlag_ptr
              jmp       #bfly
'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
mul           'Account for sign
#ifdef USE_FASTER_MULT
              abs       m1, m1 wc
              negc      m2, m2
              abs       m2, m2 wc
              'Make t2 the smaller of the 2 unsigned parameters
              mov       m3, m1
              max       m3, m2
              min       m2, m1
              'Correct the sign of the adder
              negc      m2, m2
#else
              abs       m3, m1 wc
              negc      m2, m2
#endif
              'My accumulator
              mov       m1, #0
              'Do the work
:mul_loop     shr       m3, #1 wc,wz                       'Get the low bit of t2
        if_c  add       m1, m2                             'If it was a 1, add adder to accumulator
              shl       m2, #1                             'Shift the adder left by 1 bit
        if_nz jmp       #:mul_loop                         'Continue as long as there are no more 1's
mul_ret       ret

m1            long      0
m2            long      0
m3            long      0
'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
' Get sine/cosine
'
'      quadrant: 1              2             3             4
'          angle: $0000..$07FF $0800..$0FFF $1000..$17FF $1800..$1FFF
' table index: $0000..$07FF $0800..$0001 $0000..$07FF $0800..$0001
'        mirror: +offset        -offset       +offset       -offset
'           flip: +sample       +sample       -sample       -sample
'
' On entry: sin[12..0] holds angle (0° to just under 360°)
' On exit: sin holds signed value ranging from $0000FFFF ('1') to
' $FFFF0001 ('-1')
'
getcos        add       sin, sin_90                        'For cosine, add 90°
getsin        test      sin, sin_90      wc                'Get quadrant 2|4 into c
              test      sin, sin_180     wz                'Get quadrant 3|4 into nz
              negc      sin, sin                           'If quadrant 2|4, negate offset
              or        sin, sin_table                     'OR in sin table address >> 1
              shl       sin, #1                            'Shift left to get final word address
              rdword    sin, sin                           'Read word sample from $E000 to $F000
              negnz     sin, sin                           'If quadrant 3|4, negate sample
getsin_ret
getcos_ret    ret                                          '39..54 clocks
                                                           '(variance due to HUB sync on RDWORD)
sin_90        long      $0800
sin_180       long      $1000
sin_table     long      $E000 >> 1                         'ROM sin table base shifted right
sin           long      0
'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
break         mov       var_counter, #7                    'No. of vars to dump.

              movs      :vget, #variables                  'Set vget src to variables in COG
              mov       hub_ptr, var_ptr                   'Point to data buffer in HUB

:vget         mov       vart, 0-0                          'Get a variable from COG
              wrlong    vart, hub_ptr                      'Write it to HUB buffer

              add       hub_ptr, #4                        'Next HUB location
              add       :vget, #1                          'Next COG location
              djnz      var_counter, #:vget

              mov       vart, #1                           'Indicate break
              wrlong    vart, break_cmd_ptr
:break_loop   rdlong    vart, break_cmd_ptr wz
        if_nz jmp       #:break_loop
break_ret     ret

var_counter   long      0
vart          long      0
var_ptr       long      0
break_cmd_ptr long      0
'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
variables
level         long      0
group         long      0
butterfly     long      0
b0            long      0
b1            long      0
wIndex        long      0
butterFlyMax  long      0
butterflySpan long      0
groupSize     long      0
noGroupsInit  long      FFT_SIZE / 2
noGroups      long      0
levelMax      long      0
groupIndex    long      0
groupMax      long      0
k1            long      0
k2            long      0
k3            long      0
a             long      0
b             long      0
c             long      0
d             long      0
tx            long      0
ty            long      0
index         long      0
hub_ptr       long      0
bx_ptr        long      0
by_ptr        long      0
wx_ptr        long      0
wy_ptr        long      0
runFlag_ptr   long      0
'----------------------------------------------------------------------------------------------------------------------
              fit       496
#endif
'----------------------------------------------------------------------------------------------------------------------
DAT
'The W "twiddle factor" tables.
'These are one half cycle of cos and sine respectively scaled up to +/- 4096
wx  word  4096,  4095,  4095,  4095,  4094,  4094,  4093,  4092,  4091,  4089,  4088,  4086,  4084,  4082,  4080,  4078
    word  4076,  4073,  4071,  4068,  4065,  4062,  4058,  4055,  4051,  4047,  4043,  4039,  4035,  4031,  4026,  4022
    word  4017,  4012,  4007,  4001,  3996,  3990,  3985,  3979,  3973,  3967,  3960,  3954,  3947,  3940,  3933,  3926
    word  3919,  3912,  3904,  3897,  3889,  3881,  3873,  3864,  3856,  3848,  3839,  3830,  3821,  3812,  3803,  3793
    word  3784,  3774,  3764,  3754,  3744,  3734,  3723,  3713,  3702,  3691,  3680,  3669,  3658,  3647,  3635,  3624
    word  3612,  3600,  3588,  3576,  3563,  3551,  3538,  3526,  3513,  3500,  3487,  3473,  3460,  3447,  3433,  3419
    word  3405,  3391,  3377,  3363,  3348,  3334,  3319,  3304,  3289,  3274,  3259,  3244,  3229,  3213,  3197,  3182
    word  3166,  3150,  3134,  3117,  3101,  3085,  3068,  3051,  3034,  3018,  3000,  2983,  2966,  2949,  2931,  2914
    word  2896,  2878,  2860,  2842,  2824,  2806,  2787,  2769,  2750,  2732,  2713,  2694,  2675,  2656,  2637,  2617
    word  2598,  2578,  2559,  2539,  2519,  2500,  2480,  2460,  2439,  2419,  2399,  2379,  2358,  2337,  2317,  2296
    word  2275,  2254,  2233,  2212,  2191,  2170,  2148,  2127,  2105,  2084,  2062,  2040,  2018,  1997,  1975,  1952
    word  1930,  1908,  1886,  1864,  1841,  1819,  1796,  1773,  1751,  1728,  1705,  1682,  1659,  1636,  1613,  1590
    word  1567,  1544,  1520,  1497,  1474,  1450,  1427,  1403,  1379,  1356,  1332,  1308,  1284,  1260,  1237,  1213
    word  1189,  1164,  1140,  1116,  1092,  1068,  1043,  1019,   995,   970,   946,   921,   897,   872,   848,   823
    word   799,   774,   749,   725,   700,   675,   650,   625,   601,   576,   551,   526,   501,   476,   451,   426
    word   401,   376,   351,   326,   301,   276,   251,   226,   200,   175,   150,   125,   100,    75,    50,    25
    word     0,   -25,   -50,   -75,  -100,  -125,  -150,  -175,  -200,  -226,  -251,  -276,  -301,  -326,  -351,  -376
    word  -401,  -426,  -451,  -476,  -501,  -526,  -551,  -576,  -601,  -625,  -650,  -675,  -700,  -725,  -749,  -774
    word  -799,  -823,  -848,  -872,  -897,  -921,  -946,  -970,  -995, -1019, -1043, -1068, -1092, -1116, -1140, -1164
    word -1189, -1213, -1237, -1260, -1284, -1308, -1332, -1356, -1379, -1403, -1427, -1450, -1474, -1497, -1520, -1544
    word -1567, -1590, -1613, -1636, -1659, -1682, -1705, -1728, -1751, -1773, -1796, -1819, -1841, -1864, -1886, -1908
    word -1930, -1952, -1975, -1997, -2018, -2040, -2062, -2084, -2105, -2127, -2148, -2170, -2191, -2212, -2233, -2254
    word -2275, -2296, -2317, -2337, -2358, -2379, -2399, -2419, -2439, -2460, -2480, -2500, -2519, -2539, -2559, -2578
    word -2598, -2617, -2637, -2656, -2675, -2694, -2713, -2732, -2750, -2769, -2787, -2806, -2824, -2842, -2860, -2878
    word -2896, -2914, -2931, -2949, -2966, -2983, -3000, -3018, -3034, -3051, -3068, -3085, -3101, -3117, -3134, -3150
    word -3166, -3182, -3197, -3213, -3229, -3244, -3259, -3274, -3289, -3304, -3319, -3334, -3348, -3363, -3377, -3391
    word -3405, -3419, -3433, -3447, -3460, -3473, -3487, -3500, -3513, -3526, -3538, -3551, -3563, -3576, -3588, -3600
    word -3612, -3624, -3635, -3647, -3658, -3669, -3680, -3691, -3702, -3713, -3723, -3734, -3744, -3754, -3764, -3774
    word -3784, -3793, -3803, -3812, -3821, -3830, -3839, -3848, -3856, -3864, -3873, -3881, -3889, -3897, -3904, -3912
    word -3919, -3926, -3933, -3940, -3947, -3954, -3960, -3967, -3973, -3979, -3985, -3990, -3996, -4001, -4007, -4012
    word -4017, -4022, -4026, -4031, -4035, -4039, -4043, -4047, -4051, -4055, -4058, -4062, -4065, -4068, -4071, -4073
    word -4076, -4078, -4080, -4082, -4084, -4086, -4088, -4089, -4091, -4092, -4093, -4094, -4094, -4095, -4095, -4095

wy  word     0,   -25,   -50,   -75,  -100,  -125,  -150,  -175,  -200,  -226,  -251,  -276,  -301,  -326,  -351,  -376
    word  -401,  -426,  -451,  -476,  -501,  -526,  -551,  -576,  -601,  -625,  -650,  -675,  -700,  -725,  -749,  -774
    word  -799,  -823,  -848,  -872,  -897,  -921,  -946,  -970,  -995, -1019, -1043, -1068, -1092, -1116, -1140, -1164
    word -1189, -1213, -1237, -1260, -1284, -1308, -1332, -1356, -1379, -1403, -1427, -1450, -1474, -1497, -1520, -1544
    word -1567, -1590, -1613, -1636, -1659, -1682, -1705, -1728, -1751, -1773, -1796, -1819, -1841, -1864, -1886, -1908
    word -1930, -1952, -1975, -1997, -2018, -2040, -2062, -2084, -2105, -2127, -2148, -2170, -2191, -2212, -2233, -2254
    word -2275, -2296, -2317, -2337, -2358, -2379, -2399, -2419, -2439, -2460, -2480, -2500, -2519, -2539, -2559, -2578
    word -2598, -2617, -2637, -2656, -2675, -2694, -2713, -2732, -2750, -2769, -2787, -2806, -2824, -2842, -2860, -2878
    word -2896, -2914, -2931, -2949, -2966, -2983, -3000, -3018, -3034, -3051, -3068, -3085, -3101, -3117, -3134, -3150
    word -3166, -3182, -3197, -3213, -3229, -3244, -3259, -3274, -3289, -3304, -3319, -3334, -3348, -3363, -3377, -3391
    word -3405, -3419, -3433, -3447, -3460, -3473, -3487, -3500, -3513, -3526, -3538, -3551, -3563, -3576, -3588, -3600
    word -3612, -3624, -3635, -3647, -3658, -3669, -3680, -3691, -3702, -3713, -3723, -3734, -3744, -3754, -3764, -3774
    word -3784, -3793, -3803, -3812, -3821, -3830, -3839, -3848, -3856, -3864, -3873, -3881, -3889, -3897, -3904, -3912
    word -3919, -3926, -3933, -3940, -3947, -3954, -3960, -3967, -3973, -3979, -3985, -3990, -3996, -4001, -4007, -4012
    word -4017, -4022, -4026, -4031, -4035, -4039, -4043, -4047, -4051, -4055, -4058, -4062, -4065, -4068, -4071, -4073
    word -4076, -4078, -4080, -4082, -4084, -4086, -4088, -4089, -4091, -4092, -4093, -4094, -4094, -4095, -4095, -4095
    word -4096, -4095, -4095, -4095, -4094, -4094, -4093, -4092, -4091, -4089, -4088, -4086, -4084, -4082, -4080, -4078
    word -4076, -4073, -4071, -4068, -4065, -4062, -4058, -4055, -4051, -4047, -4043, -4039, -4035, -4031, -4026, -4022
    word -4017, -4012, -4007, -4001, -3996, -3990, -3985, -3979, -3973, -3967, -3960, -3954, -3947, -3940, -3933, -3926
    word -3919, -3912, -3904, -3897, -3889, -3881, -3873, -3864, -3856, -3848, -3839, -3830, -3821, -3812, -3803, -3793
    word -3784, -3774, -3764, -3754, -3744, -3734, -3723, -3713, -3702, -3691, -3680, -3669, -3658, -3647, -3635, -3624
    word -3612, -3600, -3588, -3576, -3563, -3551, -3538, -3526, -3513, -3500, -3487, -3473, -3460, -3447, -3433, -3419
    word -3405, -3391, -3377, -3363, -3348, -3334, -3319, -3304, -3289, -3274, -3259, -3244, -3229, -3213, -3197, -3182
    word -3166, -3150, -3134, -3117, -3101, -3085, -3068, -3051, -3034, -3018, -3000, -2983, -2966, -2949, -2931, -2914
    word -2896, -2878, -2860, -2842, -2824, -2806, -2787, -2769, -2750, -2732, -2713, -2694, -2675, -2656, -2637, -2617
    word -2598, -2578, -2559, -2539, -2519, -2500, -2480, -2460, -2439, -2419, -2399, -2379, -2358, -2337, -2317, -2296
    word -2275, -2254, -2233, -2212, -2191, -2170, -2148, -2127, -2105, -2084, -2062, -2040, -2018, -1997, -1975, -1952
    word -1930, -1908, -1886, -1864, -1841, -1819, -1796, -1773, -1751, -1728, -1705, -1682, -1659, -1636, -1613, -1590
    word -1567, -1544, -1520, -1497, -1474, -1450, -1427, -1403, -1379, -1356, -1332, -1308, -1284, -1260, -1237, -1213
    word -1189, -1164, -1140, -1116, -1092, -1068, -1043, -1019,  -995,  -970,  -946,  -921,  -897,  -872,  -848,  -823
    word  -799,  -774,  -749,  -725,  -700,  -675,  -650,  -625,  -601,  -576,  -551,  -526,  -501,  -476,  -451,  -426
    word  -401,  -376,  -351,  -326,  -301,  -276,  -251,  -226,  -200,  -175,  -150,  -125,  -100,   -75,   -50,   -25
'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
'The end.

